/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.example;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import javax.servlet.AsyncContext;
import javax.servlet.AsyncEvent;
import javax.servlet.AsyncListener;
import javax.servlet.ServletException;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author ramkris
 */
@WebServlet(name="AsyncServlet", urlPatterns={"/AsyncServlet"}, asyncSupported=true)
public class AsyncServlet extends HttpServlet {

    private static class MyAsyncListener implements AsyncListener {
        boolean completed = true;
        public void onComplete(AsyncEvent event) throws IOException {
            System.out.println("MyAsyncListener: onComplete called ****");
            completed = true;
        }

        public void onTimeout(AsyncEvent event) throws IOException {
            System.out.println("MyAsyncListener: onTimeout called ****");
            completed = true;
        }

        public void onError(AsyncEvent event) throws IOException {
            System.out.println("MyAsyncListener: onError called ****");
            completed = true;
        }

        public void onStartAsync(AsyncEvent event) throws IOException {
            System.out.println("MyAsyncListener: onStartAsync called ****");
        }

    }
    
    private void invokeLongOp(final AsyncContext asyncContext, final MyAsyncListener listener) {
        ExecutorService executor = Executors.newFixedThreadPool(4);
        executor.execute(new Runnable() {

            public void run() {
                PrintWriter out = null;
                try {
                    Thread.sleep(30000L);
                    ServletResponse response = asyncContext.getResponse();
                    response.setContentType("text/html;charset=UTF-8");
                    out = response.getWriter();
                    out.println("<html>");
                    out.println("<head>");
                    out.println("<title>AsyncServlet</title>");
                    out.println("</head>");
                    out.println("<body>");
                    out.println("<h1>Finally, Servlet AsyncServlet completed Loooong op  </h1>");
                    out.println("</body>");
                    out.println("</html>");

                } catch (IOException ioe) {
                    ioe.printStackTrace();
                } catch (InterruptedException ie) {
                    ie.printStackTrace();
                } finally {
                    if (out != null) {
                        out.close();
                    }
                    synchronized(listener) {
                        if(!listener.completed)
                            asyncContext.complete();
                    }
                }
            }
        });
    }

    /**
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        
        if(request.isAsyncSupported() && !request.isAsyncStarted()) {
            AsyncContext asyncCtxt = request.startAsync(request,response);
            MyAsyncListener listener = new MyAsyncListener();
            asyncCtxt.addListener(listener);
            asyncCtxt.setTimeout(10000L);
            invokeLongOp(asyncCtxt,listener);
        }
    } 

    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    } 

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
